module mac_rx_model(
        input               rx_mac_clk,
        input               rx_mac_rst_n,

    //IP TX PORT
        input               rx_mac_valid,
        input       [ 7:0]  rx_mac_data,
        input               rx_mac_last,
        input               rx_mac_error,

        input               rx_statistics_valid,
        input       [26:0]  rx_statistics_vector,
        input               rx_pause_req,
        input       [15:0]  rx_pause_val,

    //CHECK RESULT
        output  reg         rx_error_all
    );

//**the MAC layer data receiving**//
localparam  D_MAC_ADDR_IDLE         =   48'h112233;
localparam  S_MAC_ADDR_IDLE         =   48'h445566;
localparam  MAC_TYPE_IDLE           =   16'h0800;
localparam  DATA_START_BYTE_IDLE    =   8'haa;
localparam  FRM_LENGTH_IDLE         =   16'd1200;
localparam  PAUSE_VALUE_IDLE        =   16'd274;
localparam  PAUSE_ADDRESS_IDLE      =   48'haabbcc;

localparam  IDLE                    =   2'b00;
localparam  RX_DATA                 =   2'b01;
localparam  WAIT                    =   2'b10;
localparam  RX_PAUSE                =   2'b11;

//state machine
reg [ 1:0]  cur_state;
reg [ 1:0]  next_state;

reg [111:0] shift_reg;
reg [ 7:0]  data_increase;

reg [15:0]  rx_data_cnt;

reg [143:0] shift_reg_pause;

reg [15:0]  rx_data_cnt_pause;

reg         rx_data_error;
reg         rx_fcs_error;
reg [ 1:0]  rx_length_error;
reg         rx_error;
reg         rx_other_error;

reg         rx_data_error_pause;
reg         rx_fcs_error_pause;
reg [ 1:0]  rx_length_error_pause;
reg         rx_error_pause;
reg         rx_other_error_pause;
reg         rx_pause_error;

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        cur_state   <=  IDLE;
    end else begin
        cur_state   <=  next_state;
    end
end

always @(*) begin
    case(cur_state)
        IDLE        :   next_state  =   RX_DATA;
        RX_DATA     :begin
                        if(rx_statistics_valid) begin
                            next_state  =   WAIT;
                        end else begin
                            next_state  =   RX_DATA;
                        end
                    end
        WAIT        :   next_state  =   RX_PAUSE;
        RX_PAUSE    :begin
                        if(rx_pause_req) begin
                            next_state  =   IDLE;
                        end else begin
                            next_state  =   RX_PAUSE;
                        end
                    end
        default     :   next_state  =   IDLE;
    endcase
end

//**MAC layer receive the normal data**//
always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        shift_reg   <=  112'd0;
    end else if(cur_state == RX_DATA) begin
        if(rx_mac_valid) begin
            shift_reg   <=  {shift_reg[103:0],8'd0};
        end else begin
            shift_reg   <=  shift_reg;
        end
    end else begin
        shift_reg   <=  {D_MAC_ADDR_IDLE,S_MAC_ADDR_IDLE,MAC_TYPE_IDLE};
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        data_increase   <=  8'd0;
    end else if(cur_state == RX_DATA) begin
        if(rx_data_cnt >= 16'd14) begin
            if(rx_mac_valid) begin
                data_increase   <=  data_increase + 8'd1;
            end else begin
                data_increase   <=  data_increase;
            end
        end else begin
            data_increase   <=  data_increase;
        end
    end else begin
        data_increase   <=  DATA_START_BYTE_IDLE;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_cnt <=  16'd0;
    end else if(cur_state == RX_DATA) begin
        if(rx_mac_valid) begin
            rx_data_cnt <=  rx_data_cnt + 16'd1;
        end else begin
            rx_data_cnt <=  rx_data_cnt;
        end
    end else begin
        rx_data_cnt <=  16'd0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_error   <=  1'b0;
    end else if(cur_state == RX_DATA) begin
        if(rx_mac_valid) begin
            if(rx_data_cnt < 16'd14) begin
                if(rx_mac_data != shift_reg[111:104]) begin
                    rx_data_error   <=  1'b1;
                end else begin
                    rx_data_error   <=  1'b0;
                end
            end else if(rx_data_cnt < FRM_LENGTH_IDLE) begin
                if(rx_mac_data != data_increase) begin
                    rx_data_error   <=  1'b1;
                end else begin
                    rx_data_error   <=  1'b0;
                end
            end else begin
                rx_data_error   <=  1'b0;
            end
        end else begin
            rx_data_error   <=  1'b0;
        end
    end else begin
        rx_data_error   <=  1'b0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_fcs_error    <=  1'd0;
    end else if(cur_state == RX_DATA) begin
        if(rx_statistics_valid) begin
            if(rx_statistics_vector[24]) begin
                rx_fcs_error    <=  1'd1;
            end else begin
                rx_fcs_error    <=  1'd0;
            end
        end else begin
            rx_fcs_error    <=  1'd0;
        end
    end else begin
        rx_fcs_error    <=  1'd0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_length_error <=  2'd0;
    end else if(cur_state == RX_DATA) begin
        if(rx_statistics_valid) begin
            if(FRM_LENGTH_IDLE < 16'd60) begin
                if(rx_statistics_vector[21:6] != 16'd64) begin
                    rx_length_error <=  {rx_statistics_vector[25],1'b1};
                end else begin
                    rx_length_error <=  {rx_statistics_vector[25],1'b0};
                end
            end else begin
                if(rx_statistics_vector[21:6] != (FRM_LENGTH_IDLE + 16'd4) ) begin
                    rx_length_error <=  {rx_statistics_vector[25],1'b1};
                end else begin
                    rx_length_error <=  {rx_statistics_vector[25],1'b0};
                end
            end
        end else begin
            rx_length_error <=  2'd0;
        end
    end else begin
        rx_length_error <=  2'd0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_error    <=  1'd0;
    end else if(cur_state == RX_DATA) begin
        if(rx_statistics_valid) begin
            if(rx_statistics_vector[23]) begin
                rx_error    <=  1'd1;
            end else begin
                rx_error    <=  1'd0;
            end
        end else begin
            rx_error    <=  1'd0;
        end
    end else begin
        rx_error    <=  1'd0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_other_error  <=  1'd0;
    end else if(cur_state == RX_DATA) begin
        if(rx_statistics_valid) begin
            if(rx_statistics_vector[26] || rx_statistics_vector[22]) begin
                rx_other_error  <=  1'd1;
            end else begin
                rx_other_error  <=  1'd0;
            end
        end else begin
            rx_other_error  <=  1'd0;
        end
    end else begin
        rx_other_error  <=  1'd0;
    end 
end

//**MAC layer receive the pause data**//

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        shift_reg_pause <=  144'd0;
    end else if(cur_state == RX_PAUSE) begin
        if(rx_mac_valid) begin
            shift_reg_pause <=  {shift_reg_pause[135:0],8'd0};
        end else begin
            shift_reg_pause <=  shift_reg_pause;
        end
    end else begin
        shift_reg_pause <=  {48'h0180c2000001,
                             PAUSE_ADDRESS_IDLE[7:0],PAUSE_ADDRESS_IDLE[15:8],PAUSE_ADDRESS_IDLE[23:16],PAUSE_ADDRESS_IDLE[31:24],PAUSE_ADDRESS_IDLE[39:32],PAUSE_ADDRESS_IDLE[47:40],
                             16'h8808,16'h0001,
                             PAUSE_VALUE_IDLE};
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_cnt_pause   <=  16'd0;
    end else if(cur_state == RX_PAUSE) begin
        if(rx_mac_valid) begin
            rx_data_cnt_pause   <=  rx_data_cnt_pause + 16'd1;
        end else begin
            rx_data_cnt_pause   <=  rx_data_cnt_pause;
        end
    end else begin
        rx_data_cnt_pause   <=  16'd0;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_data_error_pause <=  1'b0;
    end else if(cur_state == RX_PAUSE) begin
        if(rx_mac_valid) begin
            if(rx_data_cnt_pause < 16'd60) begin
                if(rx_mac_data != shift_reg_pause[143:136]) begin
                    rx_data_error_pause <=  1'b1;
                end else begin
                    rx_data_error_pause <=  1'b0;
                end
            end else begin
                rx_data_error_pause <=  1'b0;
            end
        end else begin
            rx_data_error_pause <=  1'b0;
        end
    end else begin
        rx_data_error_pause <=  1'b0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_fcs_error_pause  <=  1'd0;
    end else if(cur_state == RX_PAUSE) begin
        if(rx_statistics_valid) begin
            if(rx_statistics_vector[24]) begin
                rx_fcs_error_pause  <=  1'd1;
            end else begin
                rx_fcs_error_pause  <=  1'd0;
            end
        end else begin
            rx_fcs_error_pause  <=  1'd0;
        end
    end else begin
        rx_fcs_error_pause  <=  1'd0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_length_error_pause   <=  2'd0;
    end else if(cur_state == RX_PAUSE) begin
        if(rx_statistics_valid) begin
            if(rx_statistics_vector[21:6] != 16'd64) begin
                rx_length_error_pause   <=  {rx_statistics_vector[25],1'b1};
            end else begin
                rx_length_error_pause   <=  {rx_statistics_vector[25],1'b0};
            end
        end else begin
            rx_length_error_pause   <=  2'd0;
        end
    end else begin
        rx_length_error_pause   <=  2'd0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_error_pause  <=  1'd0;
    end else if(cur_state == RX_PAUSE) begin
        if(rx_statistics_valid) begin
            if(rx_statistics_vector[23]) begin
                rx_error_pause  <=  1'd1;
            end else begin
                rx_error_pause  <=  1'd0;
            end
        end else begin
            rx_error_pause  <=  1'd0;
        end
    end else begin
        rx_error_pause  <=  1'd0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_other_error_pause    <=  1'd0;
    end else if(cur_state == RX_PAUSE) begin
        if(rx_statistics_valid) begin
            if(rx_statistics_vector[26] || rx_statistics_vector[22]) begin
                rx_other_error_pause    <=  1'd1;
            end else begin
                rx_other_error_pause    <=  1'd0;
            end
        end else begin
            rx_other_error_pause    <=  1'd0;
        end
    end else begin
        rx_other_error_pause    <=  1'd0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_pause_error  <=  1'd0;
    end else if(cur_state == RX_PAUSE) begin
        if(rx_pause_req) begin
            if(rx_pause_val != PAUSE_VALUE_IDLE) begin
                rx_pause_error  <=  1'd1;
            end else begin
                rx_pause_error  <=  1'd0;
            end
        end else begin
            rx_pause_error  <=  1'd0;
        end
    end else begin
        rx_pause_error  <=  1'd0;
    end 
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_error_all    <=  1'd0;
    end else begin
        rx_error_all    <=  (rx_data_error || rx_fcs_error || (|rx_length_error) || rx_error || rx_other_error) || (rx_data_error_pause || rx_fcs_error_pause || (|rx_length_error_pause) || rx_error_pause || rx_other_error_pause || rx_pause_error);
    end 
end

endmodule
