module mac_tx_model(
        input               tx_mac_clk,
        input               tx_mac_rst_n,

    //IP TX PORT
        output  reg         tx_mac_valid,
        output  reg [ 7:0]  tx_mac_data,
        output  reg         tx_mac_last,
        output  reg         tx_mac_error,
        input               tx_mac_ready,

        output  reg         tx_pause_req,
        output  reg [15:0]  tx_pause_val,
        output  reg [47:0]  tx_pause_source_addr
    );

//**the MAC layer data sending**//
localparam  D_MAC_ADDR_IDLE         =   48'h112233;
localparam  S_MAC_ADDR_IDLE         =   48'h445566;
localparam  MAC_TYPE_IDLE           =   16'h0800;
localparam  DATA_START_BYTE_IDLE    =   8'haa;
localparam  FRM_LENGTH_IDLE         =   16'd1200;
localparam  PAUSE_VALUE_IDLE        =   16'd274;
localparam  PAUSE_ADDRESS_IDLE      =   48'haabbcc;

localparam  IDLE                    =   2'b00;
localparam  TX_DATA                 =   2'b01;
localparam  WAIT                    =   2'b10;
localparam  TX_PAUSE                =   2'b11;

//state machine
reg [ 1:0]  cur_state;
reg [ 1:0]  next_state;

reg [ 7:0]  idle_cnt;
reg         idle_end;
reg         tx_data_end;
reg [ 7:0]  wait_cnt;
reg         wait_end;

reg [111:0] shift_reg;
reg [ 7:0]  data_increase;

reg [15:0]  tx_data_cnt;

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        cur_state   <=  IDLE;
    end else begin
        cur_state   <=  next_state;
    end
end

always @(*) begin
    case(cur_state)
        IDLE        :begin
                        if(idle_end) begin
                            next_state  =   TX_DATA;
                        end else begin
                            next_state  =   IDLE;
                        end
                    end
        TX_DATA     :begin
                        if(tx_data_end) begin
                            next_state  =   WAIT;
                        end else begin
                            next_state  =   TX_DATA;
                        end
                    end
        WAIT        :begin
                        if(wait_end) begin
                            next_state  =   TX_PAUSE;
                        end else begin
                            next_state  =   WAIT;
                        end
                    end
        TX_PAUSE    :   next_state  =   IDLE;
        default     :   next_state  =   IDLE;
    endcase
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        idle_cnt    <=  8'd0;
    end else if(cur_state == IDLE)begin
        idle_cnt    <=  idle_cnt + 8'd1;
    end else begin
        idle_cnt    <=  8'd0;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        idle_end    <=  1'd0;
    end else if(cur_state == IDLE)begin
        if(idle_cnt == 8'hff) begin
            idle_end    <=  1'b1;
        end else begin
            idle_end    <=  1'b0;
        end
    end else begin
        idle_end    <=  1'd0;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_data_end <=  1'b0;
    end else if( (tx_data_cnt == FRM_LENGTH_IDLE) && tx_mac_ready)begin
        tx_data_end <=  1'b1;
    end else begin
        tx_data_end <=  1'b0;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        wait_cnt    <=  8'd0;
    end else if(cur_state == WAIT)begin
        wait_cnt    <=  wait_cnt + 8'd1;
    end else begin
        wait_cnt    <=  8'd0;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        wait_end    <=  1'd0;
    end else if(cur_state == WAIT)begin
        if(wait_cnt == 8'hff) begin
            wait_end    <=  1'b1;
        end else begin
            wait_end    <=  1'b0;
        end
    end else begin
        wait_end    <=  1'd0;
    end
end

//**MAC layer send the normal data**//
always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        shift_reg   <=  112'd0;
    end else if(cur_state == TX_DATA) begin
        if(tx_data_cnt == 16'd0) begin
            shift_reg   <=  {shift_reg[103:0],8'd0};
        end else begin
            if(tx_mac_ready) begin
                shift_reg   <=  {shift_reg[103:0],8'd0};
            end else begin
                shift_reg   <=  shift_reg;
            end
        end
    end else begin
        shift_reg   <=  {D_MAC_ADDR_IDLE,S_MAC_ADDR_IDLE,MAC_TYPE_IDLE};
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        data_increase   <=  8'd0;
    end else if(cur_state == TX_DATA) begin
        if(tx_data_cnt >= 16'd14) begin
            if(tx_mac_ready) begin
                data_increase   <=  data_increase + 8'd1;
            end else begin
                data_increase   <=  data_increase;
            end
        end else begin
            data_increase   <=  data_increase;
        end
    end else begin
        data_increase   <=  DATA_START_BYTE_IDLE;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_data_cnt <=  16'd0;
    end else if(cur_state == TX_DATA) begin
        if(tx_data_cnt == 16'd0) begin
            tx_data_cnt <=  tx_data_cnt + 16'd1;
        end else begin
            if(tx_mac_ready) begin
                tx_data_cnt <=  tx_data_cnt + 16'd1;
            end else begin
                tx_data_cnt <=  tx_data_cnt;
            end
        end
    end else begin
        tx_data_cnt <=  16'd0;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_mac_valid    <=  1'b0;
    end else if(cur_state == TX_DATA) begin
        if(tx_data_cnt == 16'd0) begin
            tx_mac_valid    <=  1'b1;
        end else if( (tx_data_cnt == FRM_LENGTH_IDLE) && tx_mac_ready) begin
            tx_mac_valid    <=  1'b0;
        end else begin
            tx_mac_valid    <=  tx_mac_valid;
        end
    end else begin
        tx_mac_valid    <=  1'b0;
    end 
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_mac_data <=  8'd0;
    end else if(cur_state == TX_DATA) begin
        if(tx_data_cnt == 16'd0) begin
            tx_mac_data <=  shift_reg[111:104];
        end else if(tx_data_cnt < 16'd14) begin
            if(tx_mac_ready) begin
                tx_mac_data <=  shift_reg[111:104];
            end else begin
                tx_mac_data <=  tx_mac_data;
            end
        end else begin
            if(tx_mac_ready) begin
                tx_mac_data <=  data_increase;
            end else begin
                tx_mac_data <=  tx_mac_data;
            end
        end
    end else begin
        tx_mac_data <=  8'd0;
    end 
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_mac_error    <=  1'b0;
    end else begin
        tx_mac_error    <=  1'b0;
    end
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_mac_last <=  1'b0;
    end else if(cur_state == TX_DATA) begin
        if( (tx_data_cnt == (FRM_LENGTH_IDLE - 16'd1) ) && tx_mac_ready) begin
            tx_mac_last  <=  1'b1;
        end else if( (tx_data_cnt == FRM_LENGTH_IDLE) && tx_mac_ready) begin
            tx_mac_last  <=  1'b0;
        end else begin
            tx_mac_last  <=  tx_mac_last;
        end
    end else begin
        tx_mac_last  <=  1'b0;
    end 
end

//**MAC layer send the pause data**//
always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_pause_req    <=  1'b0;
    end else if(cur_state == TX_PAUSE) begin
        tx_pause_req    <=  1'b1;
    end else begin
        tx_pause_req    <=  1'b0;
    end 
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_pause_val    <=  16'd0;
    end else if(cur_state == TX_PAUSE) begin
        tx_pause_val    <=  PAUSE_VALUE_IDLE;
    end else begin
        tx_pause_val    <=  16'd0;
    end 
end

always @(posedge tx_mac_clk or negedge tx_mac_rst_n) begin
    if(~tx_mac_rst_n) begin
        tx_pause_source_addr    <=  48'd0;
    end else if(cur_state == TX_PAUSE) begin
        tx_pause_source_addr    <=  PAUSE_ADDRESS_IDLE;
    end else begin
        tx_pause_source_addr    <=  48'd0;
    end 
end

endmodule